//
// ECE264 PA 4
// Yung-Hsiang Lu, ECE Purdue University
// Spring 2009
//
#include "rectlist.h"
#include <QFile>
#include <iostream>
using namespace std;
Rectangle::Rectangle(int id, bool r, int x, int y, int w, int h,
		     QString input, QString location): 
  r_id(id), r_rotate(r), r_rect(x, y, w, h),
  r_input(input), r_location(location)
{ 
}

void Rectangle::print()
{
  cout << "id= " << r_id 
       << " x= " << r_rect.x()
       << " y= " << r_rect.y()
       << " w= " << r_rect.width() 
       << " h= " << r_rect.height() 
       << " rotate= " << r_rotate << endl;
}
RectList::RectList(const char * filename1, const char * filename2)
{
  QFile inFile(filename1);
  QFile locFile(filename2);
  rl_maxX = 0;
  rl_maxY = 0;
  if (! inFile.open(QIODevice::ReadOnly | QIODevice::Text))
    {
      cout << "cannot open " << filename1 << endl;
      return;
    }
  if (! locFile.open(QIODevice::ReadOnly | QIODevice::Text))
    {
      cout << "cannot open " << filename1 << endl;
      return;
    }
  int rectID = 0;
  while ((! inFile.atEnd()) && (! locFile.atEnd()))
    {
      QString inLine = inFile.readLine();
      QString locLine = locFile.readLine();
      QStringList inStrList = inLine.split(" ");
      QStringList locStrList = locLine.split(" ");
      int inID = inStrList.at(0).toInt();
      int inWidth = inStrList.at(1).toInt();
      int inHeight = inStrList.at(2).toInt();
      // ignore the area provided in the input file
      int locID = locStrList.at(0).toInt();
      int locX = locStrList.at(1).toInt();
      int locY = locStrList.at(2).toInt();
      bool rotate = (locStrList.at(3).toInt() == 1);
      if (checkValue(rectID, inID, inWidth, inHeight,
		     locID, locX, locY, rotate) == false)
	{ return; }
      int rectX = locX + inWidth; /* upper right corner */
      int rectY = locY + inHeight;
      if (rotate == true)
	{
	  rectX = locX + inHeight;
	  rectY = locY + inWidth;
	}
      if (rl_maxX < rectX) { rl_maxX = rectX; }
      if (rl_maxY < rectY) { rl_maxY = rectY; }
      /*
      cout << "add rectangle id " << rectID 
	   << " locX " << locX
	   << " locY " << locY << endl;
      */
      rl_list.append(Rectangle(rectID, rotate, 
			       locX, locY, 
			       inWidth, inHeight,
			       inLine, locLine));
      rectID ++;
    }
  if ((! inFile.atEnd()) || (! locFile.atEnd()))
    {
      cout << "files have different numbers of rectangles" << endl;
      return;
    }
  inFile.close();
  locFile.close();
  if (checkOverlap() == false)
    {
      cout << "*************************************" << endl;
      cout << "           maxX = " << rl_maxX 
	   << " maxY = " << rl_maxY << endl;
      cout << "           area = " << rl_maxX * rl_maxY << endl;
      cout << "*************************************" << endl;
    }
}

bool RectList::checkValue(int rectID,
			  int inID, int inWidth, int inHeight, 
			  int locID, int locX, int locY, int rotate)
{
  if ((inID != rectID) || (locID != rectID))
    {
      cout << "wrong rectangle ID " << rectID 
	   << " " << inID << " " << locID << endl;
      return false;
    }
  if ((inWidth <= 0) || (inHeight <= 0))
    {
      cout << "negative size" << endl;
      return false;
    }
  if ((locX < 0) || (locY < 0))
    {
      cout << "must be in the first quadrant" << endl;
      return false;
    }
  if ((rotate != 0) && (rotate != 1))
    {
      cout << "either rotate (1) or not (0), no other value" << endl;
      return false;
    }
  return true;
}

bool RectList::checkOverlap()
{
  // true means overlap
  if (rl_list.size() <= 1) { return false; }
  QList<Rectangle>::iterator iter1 = rl_list.begin();
  QList<Rectangle>::iterator iter2 = rl_list.begin();
  QList<Rectangle>::iterator secondLast = rl_list.end();
  secondLast --;
  bool overlap = false;
  while (iter1 != secondLast)
    {
      iter2 = iter1;
      iter2 ++;
      while (iter2 != rl_list.end())
	{
	  if ((*iter1).r_rect.intersects((*iter2).r_rect) == true)
	    {
	      QRect intersectRect = 
		(*iter1).r_rect.intersected((*iter2).r_rect);
	      cout << "rectangle " << (*iter1).r_id 
		   << " overlaps with "<< (*iter2).r_id 
		   << " at (" << intersectRect.x() 
		   << ", " << intersectRect.y() 
		   << ") width " << intersectRect.width() 
		   << " height " << intersectRect.height() 
		   << endl;
	      overlap = true;
	    }
	  iter2 ++;
	}
      iter1 ++;
    }
  return overlap;
}

void RectList::print()
{
  QList<Rectangle>::iterator iter = rl_list.begin();
  while (iter != rl_list.end())
    {
      (*iter).print();
      iter ++;
    }
}
